/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------
#include "Epwm.h"
#include "cpu.h"
#include "gpio.h"

//-----------------------------------------------------------------------------
//	Variables
//-----------------------------------------------------------------------------
volatile struct PWM3phConfiguration pwmConf;
volatile struct PWM3phConfiguration pwmAConf;
volatile struct PWM3phConfiguration pwmBConf;
extern const Uint16 _offsetPWM;
volatile struct EPWM_REGS *ePWM[] = {&EPwm1Regs, &EPwm2Regs, &EPwm3Regs, &EPwm4Regs,
&EPwm5Regs, &EPwm6Regs};
volatile Uint16* cmpPtr[PWM_OUTPUTS_SIZE];
int MEP_ScaleFactor[PWM_CH] = {0,0,0,0,0};  // Scale factor values for ePWM1-4
int MEP_SF1, MEP_SF2, MEP_SF3, MEP_SF4, MEP_SF5, MEP_SF6;


//-----------------------------------------------------------------------------
//	Consts
//-----------------------------------------------------------------------------

/*prescalerMat =
[
	1           2           4           6           8          10          12
	2           4           8          12          16          20          24
	4           8          16          24          32          40          48
	8          16          32          48          64          80          96
	32          64         128         192         256         320         384
	64         128         256         384         512         640         768
	128         256         512         768        1024        1280        1536]*/
	
// prescaler values for CLKDIV
static const Uint16 clkMat[] = {1, 2, 4, 8, 32, 64, 128};
// prescaler values for HSPCLKDIV	
static const Uint16 hspClkMat[] = {1, 2, 4, 6, 8, 10, 12};
// index of HSPCLKDIV that allows for obtaining unique prescaler values (col indx in prescalerMat)
static const Uint16 hspClkIndxMat[] = {0, 1, 2, 3, 4, 5, 6, 4, 5, 6, 4, 5, 6,
	4, 5, 6, 2, 3, 4, 5, 6, 4, 5, 6, 4, 5, 6};
// index of CLKDIV that allows for obtaining unique prescaler values (row indx in prescalerMat)
static const Uint16 lspClkIndxMat[] = {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 2, 2, 2,
	3, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5, 6, 6, 6};

//-----------------------------------------------------------------------------
//    Functions
//-----------------------------------------------------------------------------

void BindPWMOutputs(void)
{
	int i;
	// associate pointers to compare registers on each pwm outputs
	cmpPtr[PWM1+xA] = &(EPwm1Regs.CMPA.half.CMPA);
	cmpPtr[PWM1+xB] = &(EPwm1Regs.CMPB);
	cmpPtr[PWM2+xA] = &(EPwm2Regs.CMPA.half.CMPA);
	cmpPtr[PWM2+xB] = &(EPwm2Regs.CMPB);
	cmpPtr[PWM3+xA] = &(EPwm3Regs.CMPA.half.CMPA);
	cmpPtr[PWM3+xB] = &(EPwm3Regs.CMPB);
	cmpPtr[PWM4+xA] = &(EPwm4Regs.CMPA.half.CMPA);
	cmpPtr[PWM4+xB] = &(EPwm4Regs.CMPB);
	cmpPtr[PWM5+xA] = &(EPwm5Regs.CMPA.half.CMPA);
	cmpPtr[PWM5+xB] = &(EPwm5Regs.CMPB);
	cmpPtr[PWM6+xA] = &(EPwm6Regs.CMPA.half.CMPA);
	cmpPtr[PWM6+xB] = &(EPwm6Regs.CMPB);
	//  MEP_ScaleFactor variables intialization for SFO library functions
    for(i=0;i<PWM_CH;i++){
        MEP_ScaleFactor[i] =0;
    }

	//  MEP_ScaleFactor variables intialization using SFO_MepDis_V5 library function.
   	for(i=1;i<PWM_CH;i++){
	    while ( SFO_MepDis_V5(i) == SFO_INCOMPLETE ); //returns "0" when cal. incomplete for channel
   	}

	// 	Initialize a common seed variable MEP_ScaleFactor[0] required for all SFO functions
    MEP_ScaleFactor[0] = MEP_ScaleFactor[1];

}

// ----------------------------------------------------------------------------

/// Calculates prescaler bits for a given pwm period
static inline void CalculatePrescalerBits(Uint16 tpwm_us,Uint16* hspClkDiv, Uint16* clkDiv)
{
	// max number of unique prescaler values
	const Uint16 maxIndx = 28;
	const Uint32 initVal = (Uint32)tpwm_us*CpuInMhz();
	Uint32 ctrVal = initVal;
	Uint16 indx;
	for (indx = 0;indx<maxIndx;indx++) {
		ctrVal = initVal/(hspClkMat[hspClkIndxMat[indx]]*
			clkMat[lspClkIndxMat[indx]]);
		if(ctrVal<=65535)
			break;
	}
	*hspClkDiv = hspClkIndxMat[indx];
	*clkDiv = lspClkIndxMat[indx];	
}

// ----------------------------------------------------------------------------

void InitPWMSide(ePWMenum PWM, sideEnum side, PWMMODULATION modulation, 
	SWITCHLOGIC switchLogic)
{
    volatile struct EPWM_REGS* pwmPtr = ePWM[PWM];
    if(side == xA){
		cmpPtr[PWM+xA] = &(pwmPtr->CMPA.half.CMPA);
		// Counter compare submodule	
		pwmPtr->CMPCTL.bit.SHDWAMODE = CC_SHADOW;		// shadow CMPR register	
		pwmPtr->CMPCTL.bit.LOADAMODE = modulation; 	// load on (CC_CTR_PRD for period, CC_CTR_ZERO_PRD for both)
		// Action qualifier submodule
		pwmPtr->AQCTLA.bit.CAU = ~switchLogic; 		// CAU: counter equals CMPA on up-count
		pwmPtr->AQCTLA.bit.CAD = switchLogic; 		// CAD: counter equals CMPA on down-count	
    }
    else{
		cmpPtr[PWM+xB] = &(pwmPtr->CMPB);	
		pwmPtr->CMPCTL.bit.SHDWBMODE = CC_SHADOW;	
		pwmPtr->CMPCTL.bit.LOADBMODE = modulation; 	// load on ...	
		// Action qualifier submodule	
		pwmPtr->AQCTLB.bit.CBU = ~switchLogic; 		// CBU: counter equals CMPB on up-count
		pwmPtr->AQCTLB.bit.CBD = switchLogic; 		// CBD: counter equals CMPB on down-count	
    }
}

// ----------------------------------------------------------------------------

void InitSinglePWMOutput(void(*interruptFCN)(void), ePWMenum PWM, sideEnum side, Uint16 tpwm_us,
	PHSEN phSen, PWMMODE pwmMode, SYNCOSEL syncMode, PWMCARRIER pwmCarrier, 
	PWMMODULATION modulation, INTSEL intSel, SWITCHLOGIC switchLogic)
{
    volatile struct EPWM_REGS* pwmPtr = ePWM[PWM];
	Uint16 PRRSC = 1;
	Uint16 hspClkDiv, clkDiv;
	// bind pwm outputs	
	InitEPWMGpio(PWM);
	// enable PWM cloks
	EALLOW;
	SysCtrlRegs.PCLKCR1.all |= 1 << PWM;
   	
   	// disable clock
   	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
 	EDIS;
	// Time-base submodule
	// PABLO: These prescalers should be maximized to increase PWM resolution
	CalculatePrescalerBits(tpwm_us,&hspClkDiv, &clkDiv);	
	pwmPtr->TBCTL.bit.CLKDIV 	 = clkDiv;//TB_DIV1;				// Clock ratio to SYSCLKOUT
	pwmPtr->TBCTL.bit.HSPCLKDIV = hspClkDiv;//TB_DIV1;	
	PRRSC *= (1 << pwmPtr->TBCTL.bit.CLKDIV);
	if(pwmPtr->TBCTL.bit.HSPCLKDIV != TB_DIV1)
		PRRSC *= 2*pwmPtr->TBCTL.bit.HSPCLKDIV;
	// for TB_COUNT_UP: TPWM = (TBPRD + 1) x TTBCLK
	// for TB_COUNT_UPDOWN: TPWM = 2 x TBPRD x TTBCLK
    pwmPtr->TBCTL.bit.CTRMODE = pwmCarrier;				// counter mode
    pwmPtr->TBCTL.bit.PRDLD = TB_IMMEDIATE;;			// inmediate TBPRD register
	// PABLO: Just TB_COUNT_UPDOWN and TB_COUNT_UP are suported 
	if(pwmCarrier == TB_COUNT_UPDOWN)
	    pwmPtr->TBPRD = US2CLOCK(tpwm_us, PRRSC)/2; 	// Period (up/down mode)
	else if(pwmCarrier == TB_COUNT_UP)
		pwmPtr->TBPRD = US2CLOCK(tpwm_us, PRRSC)-1;
	pwmPtr->TBCTL.bit.PHSEN = phSen;         	// Master/Slave
    pwmPtr->TBCTL.bit.SYNCOSEL = syncMode; 	// Synchronization mode
	InitPWMSide(PWM, side, modulation, switchLogic);
    
    
    // pwmPtr->AQCTLA.bit.PRD = switchLogic;		// clear on period
    // pwmPtr->AQCTLB.bit.PRD = switchLogic;		// clear on period
	// configure HRPWM extension
	if(pwmMode == HRPWM_MODE){
		EALLOW;
		pwmPtr->HRCNFG.all = 0x0;
		pwmPtr->HRCNFG.bit.EDGMODE = HR_FEP;         // Control falling edge position
		pwmPtr->HRCNFG.bit.CTLMODE = HR_CMP;         // CMPAHR controls the MEP.
		pwmPtr->HRCNFG.bit.HRLOAD = HR_CTR_ZERO;     // Shadow load on CTR=Ze
		// MEP_SF = 66*256;
		// Start with typical Scale Factor
		// value for 100 MHz.
		// Use SFO functions to update MEP_SF dynamically.
		EDIS;
	}
	if(interruptFCN !=0){
		pwmPtr->ETSEL.bit.INTSEL = intSel;      		// Select INT on Zero or Period event
		pwmPtr->ETSEL.bit.INTEN = 1;                 	// Enable INT
		pwmPtr->ETPS.bit.INTPRD = ET_1ST;            	// Generate INT on 1st event
        // enable pwm interrupts on MASTERPWM
        EALLOW;  
		*(&PieVectTable.EPWM1_INT + PWM) = interruptFCN;
   		EDIS;
		// Enable CPU INT3 which is connected to EPWM1-6 INT:
   		IER |= M_INT3;
		// PIE enable
		PieCtrlRegs.PIEIER3.all |= 1 << PWM;
	}
}

// ----------------------------------------------------------------------------

void InitComplementaryPWMOutput(void(*interruptFCN)(void), ePWMenum PWM, Uint16 tpwm_us,
	Uint16 db_us, PHSEN phSen, PWMMODE pwmMode, PWMCARRIER pwmCarrier, PWMMODULATION modulation, 
	INTSEL intSel, SWITCHLOGIC switchLogic)
{
    volatile struct EPWM_REGS* pwmPtr = ePWM[PWM];
	InitEPWMGpio(PWM);
	// bind pwm outputs
	cmpPtr[PWM+xA] = &(pwmPtr->CMPA.half.CMPA);
	// enable PWM cloks
	EALLOW;
	SysCtrlRegs.PCLKCR1.all |= 1 << PWM;
   	
   	// disable clock
   	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
 	EDIS;
    ConfigurePWMLeg(pwmPtr, phSen, tpwm_us, db_us, pwmCarrier, modulation, switchLogic);
	// configure HRPWM extension
	if(pwmMode == HRPWM_MODE){
		EALLOW;
		pwmPtr->HRCNFG.all = 0x0;
		pwmPtr->HRCNFG.bit.EDGMODE = HR_FEP;         // Control falling edge position
		pwmPtr->HRCNFG.bit.CTLMODE = HR_CMP;         // CMPAHR controls the MEP.
		pwmPtr->HRCNFG.bit.HRLOAD = HR_CTR_ZERO;     // Shadow load on CTR=Ze
		// MEP_SF = 66*256;
		// Start with typical Scale Factor
		// value for 100 MHz.
		// Use SFO functions to update MEP_SF dynamically.
		EDIS;
	}
	if(interruptFCN !=0){
		pwmPtr->ETSEL.bit.INTSEL = intSel;      		// Select INT on Zero or Period event
		pwmPtr->ETSEL.bit.INTEN = 1;                 	// Enable INT
		pwmPtr->ETPS.bit.INTPRD = ET_1ST;            	// Generate INT on 1st event
        // enable pwm interrupts on MASTERPWM
        EALLOW;  
		*(&PieVectTable.EPWM1_INT + PWM) = interruptFCN;
   		EDIS;
		// Enable CPU INT3 which is connected to EPWM1-6 INT:
   		IER |= M_INT3;
		// cambiar la habilitacin del PIE en funcin de la entrada
		PieCtrlRegs.PIEIER3.all |= 1 << PWM;
	}
}

// ----------------------------------------------------------------------------

void Init3phPWM(void(*interruptFCN)(void), EPWMX epwmX, Uint16 tpwm_us, 
	Uint16 db_us,  PWMCARRIER pwmCarrier, PWMMODULATION modulation, INTSEL intSel,
	_iq vBus, ePWMSOC soc, SOCTrigger trigger, SWITCHLOGIC switchLogic)
{
    volatile struct EPWM_REGS* pwm1Ptr = 0;
    volatile struct EPWM_REGS* pwm2Ptr = 0; 
    volatile struct EPWM_REGS* pwm3Ptr = 0;
    pwmConf.epwmX = epwmX;
    pwmConf.t_pwm = tpwm_us;
    if(epwmX == EPWMA){
        InitEPWMGpio(PWM1);
        InitEPWMGpio(PWM2);
       	InitEPWMGpio(PWM3);
        pwm1Ptr = &EPwm1Regs;
        pwm2Ptr = &EPwm2Regs;
        pwm3Ptr = &EPwm3Regs;
    }
    else{
        InitEPWMGpio(PWM4);
        InitEPWMGpio(PWM5);
       	InitEPWMGpio(PWM6);
        pwm1Ptr = &EPwm4Regs;
        pwm2Ptr = &EPwm5Regs;
        pwm3Ptr = &EPwm6Regs;
    }
    // configure comparison registers
    pwmConf.cmp1 = &(pwm1Ptr->CMPA.half.CMPA);
    pwmConf.cmp2 = &(pwm2Ptr->CMPA.half.CMPA);
    pwmConf.cmp3 = &(pwm3Ptr->CMPA.half.CMPA);
    // init peripheral clocks
    InitPWMPeripheralClocks();
    // configure PWM modules for 3ph systems
	// disable clock
   	EALLOW;
   	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
   	EDIS;
    ConfigurePWMLeg(pwm1Ptr, MASTERPWM, tpwm_us, db_us, pwmCarrier, modulation, switchLogic);
    ConfigurePWMLeg(pwm2Ptr, SLAVEPWM, tpwm_us, db_us, pwmCarrier, modulation, switchLogic);
    ConfigurePWMLeg(pwm3Ptr, SLAVEPWM, tpwm_us, db_us, pwmCarrier, modulation, switchLogic);
    // configure interrupts and SOC
	pwm1Ptr->ETSEL.bit.INTSEL = intSel;      		// Select INT on Zero or Period event
	pwm1Ptr->ETSEL.bit.INTEN = 1;                 	// Enable INT
	pwm1Ptr->ETPS.bit.INTPRD = ET_1ST;            	// Generate INT on 1st event
	AddEventTrigger(pwm1Ptr, soc, trigger);
	/*if(soc == SOCA){
   		pwm1Ptr->ETSEL.bit.SOCAEN = 1;        		// Enable SOC on A group
		pwm1Ptr->ETSEL.bit.SOCASEL = 1;       		// Select SOC from time-base counter equal to zero.
		pwm1Ptr->ETPS.bit.SOCAPRD = 1;        		// Generate pulse on 1st event
	}
	else if(soc == SOCB){
	   	pwm1Ptr->ETSEL.bit.SOCBEN = 1;        		// Enable SOC on B group
		pwm1Ptr->ETSEL.bit.SOCBSEL = 1;       		// Select SOC from time-base counter equal to zero.
		pwm1Ptr->ETPS.bit.SOCBPRD = 1;        		// Generate pulse on 1st event
	}*/  
    if(interruptFCN != 0){
		// Enable CPU INT3 which is connected to EPWM1-6 INT:
   		IER |= M_INT3;
		if(epwmX == EPWMA){
			// enable pwm interrupts on MASTERPWM
        	EALLOW;  
   			PieVectTable.EPWM1_INT = interruptFCN;
   			EDIS;
		   	PieCtrlRegs.PIEIER3.bit.INTx1 = 1;	// EPMW1
		}
		else{
			 // enable pwm interrupts on MASTERPWM
        	EALLOW;  
   			PieVectTable.EPWM4_INT = interruptFCN;
   			EDIS;
			PieCtrlRegs.PIEIER3.bit.INTx4 = 1;	// EPWM4
		}
    }
    // store configuration in pwmConf struct
	pwmConf.OFFSETPWM = pwm1Ptr->TBPRD >> 1;
	pwmConf.OFFSETPWMDIVVBUS = _IQdiv(_IQ(pwmConf.OFFSETPWM),vBus >> 1);
	pwmConf.vBus = vBus;
	// copy configuration to A/B configuration structures
	if(epwmX == EPWMA)
		pwmAConf = pwmConf;
	else
		pwmBConf = pwmConf;
}

// ----------------------------------------------------------------------------

void ConfigurePWMLeg(volatile struct EPWM_REGS* pwmPtr, PHSEN master, 
	Uint16 tpwm_us, Uint16 db_us, PWMCARRIER pwmCarrier, PWMMODULATION modulation, 
	unsigned int polSel)
{
	// TB Prescalers
	// TBCLK = SYSCLKOUT / (HSPCLKDIV X CLKDIV)
	Uint16 PRRSC = 1;
	Uint16 hspClkDiv, clkDiv;	
	// PABLO: These prescalers should be maximized to increase PWM resolution
	CalculatePrescalerBits(tpwm_us,&hspClkDiv, &clkDiv);
	pwmPtr->TBCTL.bit.CLKDIV 	 = clkDiv;//TB_DIV1;//TB_DIV4;				// Clock ratio to SYSCLKOUT
	pwmPtr->TBCTL.bit.HSPCLKDIV = hspClkDiv;//TB_DIV1;//TB_DIV4;
	PRRSC *= (1 << pwmPtr->TBCTL.bit.CLKDIV);
	if(pwmPtr->TBCTL.bit.HSPCLKDIV != TB_DIV1)
		PRRSC *= 2*pwmPtr->TBCTL.bit.HSPCLKDIV;
	// Time-base submodule
	// for TB_COUNT_UP: TPWM = (TBPRD + 1) x TTBCLK
	// for TB_COUNT_UPDOWN: TPWM = 2 x TBPRD x TTBCLK
    pwmPtr->TBCTL.bit.CTRMODE = pwmCarrier;				// Sawtooth/triangle carrier
	if(pwmPtr->TBCTL.bit.CTRMODE == TB_COUNT_UPDOWN)	 
	    pwmPtr->TBPRD = US2CLOCK(pwmConf.t_pwm, PRRSC)/2; 	// Period (up/down mode)
	else
		pwmPtr->TBPRD = US2CLOCK(pwmConf.t_pwm, PRRSC)-1;
    pwmPtr->TBPHS.half.TBPHS = 0; 						// Set Phase register to zero
    pwmPtr->TBCTL.bit.PHSEN = master;					// MASTER (TB_DISABLE) or SLAVE (TB_ENABLE) module
    pwmPtr->TBCTL.bit.PRDLD = CC_SHADOW;				// shadow TBPRD register
    pwmPtr->TBCTL.bit.SYNCOSEL = TB_CTR_ZERO; 			// Sync down-stream module
    // Counter compare submodule
    pwmPtr->CMPCTL.bit.SHDWAMODE = CC_SHADOW;			// shadow CMPR register
    pwmPtr->CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    pwmPtr->CMPCTL.bit.LOADAMODE = modulation; 		// load on CTR=Zero (CC_CTR_PRD for period, CC_CTR_ZERO_PRD for both)
    pwmPtr->CMPCTL.bit.LOADBMODE = modulation; 		// load on CTR=Zero
    // Action qualifier submodule (set actions for EPWMA) 
    pwmPtr->AQCTLA.bit.CAU = ~polSel;//AQ_SET; 					// CAU: counter equals A on up-count
    pwmPtr->AQCTLA.bit.CAD = polSel;//AQ_CLEAR;					// CAD: counter equals D on down-count
	// Dead-band submodule
    pwmPtr->DBCTL.bit.OUT_MODE = DB_FULL_ENABLE; 		// enable Dead-band module
    pwmPtr->DBCTL.bit.POLSEL = polSel; 					// polarity selection for dead time
	// 10 bit register --> maximum value 1024
    pwmPtr->DBFED = US2CLOCK(db_us, PRRSC); 			// FED
    pwmPtr->DBRED = US2CLOCK(db_us, PRRSC); 			// RED
}

// ----------------------------------------------------------------------------

// spru791a.pdf, pag. 109
void AddEventTrigger(volatile struct EPWM_REGS* pwmPtr, 
	ePWMSOC soc, SOCTrigger trigger)
{
	if(soc == SOCA){
   		pwmPtr->ETSEL.bit.SOCAEN = 1;        		// Enable SOC on A group
		pwmPtr->ETSEL.bit.SOCASEL = trigger;       	// Select SOC from time-base counter equal to zero.
		pwmPtr->ETPS.bit.SOCAPRD = 1;        		// Generate pulse on 1st event
	}
	else if(soc == SOCB){
	   	pwmPtr->ETSEL.bit.SOCBEN = 1;        		// Enable SOC on B group
		pwmPtr->ETSEL.bit.SOCBSEL = trigger;       	// Select SOC from time-base counter equal to zero.
		pwmPtr->ETPS.bit.SOCBPRD = 1;        		// Generate pulse on 1st event
	}
}

// ----------------------------------------------------------------------------

void InitPWMPeripheralClocks(void)
{
    EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;		 // Disable TBCLK within the ePWM
    if(pwmConf.epwmX == EPWMA){
        SysCtrlRegs.PCLKCR1.bit.EPWM1ENCLK = 1;  // ePWM1
   		SysCtrlRegs.PCLKCR1.bit.EPWM2ENCLK = 1;  // ePWM2
   		SysCtrlRegs.PCLKCR1.bit.EPWM3ENCLK = 1;  // ePWM3
    }
    else{
        SysCtrlRegs.PCLKCR1.bit.EPWM4ENCLK = 1;  // ePWM4
        SysCtrlRegs.PCLKCR1.bit.EPWM5ENCLK = 1;  // ePWM5
       	SysCtrlRegs.PCLKCR1.bit.EPWM6ENCLK = 1;  // ePWM6
    }
	// ... if needed configure pre-scaler here
    SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;       // Enable TBCLK within the ePWM
    EDIS;
}

// ----------------------------------------------------------------------------

void InitPWMAPeripheralClocks(void)
{
    EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;		 // Disable TBCLK within the ePWM
	SysCtrlRegs.PCLKCR1.bit.EPWM1ENCLK = 1;  // ePWM1
	SysCtrlRegs.PCLKCR1.bit.EPWM2ENCLK = 1;  // ePWM2
	SysCtrlRegs.PCLKCR1.bit.EPWM3ENCLK = 1;  // ePWM3
	// ... if needed configure pre-scaler here
    SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;       // Enable TBCLK within the ePWM
    EDIS;
}

// ----------------------------------------------------------------------------

void InitPWMBPeripheralClocks(void)
{
    EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;		 // Disable TBCLK within the ePWM
    SysCtrlRegs.PCLKCR1.bit.EPWM4ENCLK = 1;  // ePWM4
    SysCtrlRegs.PCLKCR1.bit.EPWM5ENCLK = 1;  // ePWM5
    SysCtrlRegs.PCLKCR1.bit.EPWM6ENCLK = 1;  // ePWM6
	// ... if needed configure pre-scaler here
    SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;       // Enable TBCLK within the ePWM
    EDIS;
}

// ----------------------------------------------------------------------------

void InitEPWMGpio(ePWMenum ePWM)
{
    // GPIO pins are configured to: 
    //	- be managed by the EPWM (GpioCtrlRegs.GPAMUX1.bit.GPIOx = 1)
    //	- have the pullup resistor enabled (GpioCtrlRegs.GPAPUD.bit.GPIOx)
    switch(ePWM){
        case PWM1:
	    ConfigureGPIOPort(_GPIO0, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
	    ConfigureGPIOPort(_GPIO1, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
            break;
        case PWM2:
	    ConfigureGPIOPort(_GPIO2, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
	    ConfigureGPIOPort(_GPIO3, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);	    
            break;
        case PWM3:
	    ConfigureGPIOPort(_GPIO4, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
	    ConfigureGPIOPort(_GPIO5, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
            break;
        case PWM4:
	    ConfigureGPIOPort(_GPIO6, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
	    ConfigureGPIOPort(_GPIO7, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);	    
            break;
        case PWM5:
	    ConfigureGPIOPort(_GPIO8, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
	    ConfigureGPIOPort(_GPIO9, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);	    
            break;
        case PWM6:
	    ConfigureGPIOPort(_GPIO10, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
	    ConfigureGPIOPort(_GPIO11, GPIO_OUTPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);
            break;
    }
}

//-----------------------------------------------------------------------------

void InitTzGpio(void)
{  
    EALLOW;

    // Enable internal pull-up for the selected pins
    // Pull-ups can be enabled or disabled by the user. 
    // This will enable the pullups for the specified pins.
    // Comment out other unwanted lines.
    GpioCtrlRegs.GPAPUD.bit.GPIO12 = 0;    // Enable pull-up on GPIO12 (TZ1)
    GpioCtrlRegs.GPAPUD.bit.GPIO13 = 0;    // Enable pull-up on GPIO13 (TZ2)
    GpioCtrlRegs.GPAPUD.bit.GPIO14 = 0;    // Enable pull-up on GPIO14 (TZ3)
    GpioCtrlRegs.GPAPUD.bit.GPIO15 = 0;    // Enable pull-up on GPIO15 (TZ4)
    GpioCtrlRegs.GPAPUD.bit.GPIO16 = 0;    // Enable pull-up on GPIO16 (TZ5)
    GpioCtrlRegs.GPAPUD.bit.GPIO17 = 0;    // Enable pull-up on GPIO17 (TZ6) 

    // Set qualification for selected pins to asynch only
    // Inputs are synchronized to SYSCLKOUT by default.  
    // This will select asynch (no qualification) for the selected pins.
    // Comment out other unwanted lines.

    GpioCtrlRegs.GPAQSEL1.bit.GPIO12 = 3;  // Asynch input GPIO12 (TZ1)
    GpioCtrlRegs.GPAQSEL1.bit.GPIO13 = 3;  // Asynch input GPIO13 (TZ2)
    GpioCtrlRegs.GPAQSEL1.bit.GPIO14 = 3;  // Asynch input GPIO14 (TZ3)
    GpioCtrlRegs.GPAQSEL1.bit.GPIO15 = 3;  // Asynch input GPIO15 (TZ4)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO16 = 3;  // Asynch input GPIO16 (TZ5)
    GpioCtrlRegs.GPAQSEL2.bit.GPIO17 = 3;  // Asynch input GPIO17 (TZ6) 


    // Configure TZ pins using GPIO regs
    // This specifies which of the possible GPIO pins will be TZ functional pins.
    // Comment out other unwanted lines.   
    GpioCtrlRegs.GPAMUX1.bit.GPIO12 = 1;  // Configure GPIO12 as TZ1
    GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 1;  // Configure GPIO13 as TZ2
    GpioCtrlRegs.GPAMUX1.bit.GPIO14 = 1;  // Configure GPIO14 as TZ3
    GpioCtrlRegs.GPAMUX1.bit.GPIO15 = 1;  // Configure GPIO15 as TZ4
    GpioCtrlRegs.GPAMUX2.bit.GPIO16 = 3;  // Configure GPIO16 as TZ5
    GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 3;  // Configure GPIO17 as TZ6               

    EDIS;
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
