/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	FilterAndRotations.h
 Project	:  	TIMotorLIB
 TI Doc:		na 
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \brief 		Macros for filter, rotations and discrete PI, PID regulators 
 \date 			2007
******************************************************************************/

#ifndef FILTERANDROTATIONS_H
#define FILTERANDROTATIONS_H

/// \def LIBRARY_BUILD error control when using library
#ifndef LIBRARY_BUILD
	#ifndef DTDIV2
		#error DTDIV2 (sampling rate divide by 2) MUST be defined to use this header
	#endif
#endif

#include "Constants.h"

/// \defgroup ROTATE_QDDATA define positive & negative rotation of qddata

/*! \def ROTATE_POS(in,out,t) 
 * \ingroup ROTATE_QDDATA
 * define positive rotation of qddata (rotate to synchronous coordinates out=in*exp(j*t))
 *
 * \a in data in the original reference frame.
 * 
 * \a out data in the rotated reference frame.
 * 
 * \a t angle of rotation.
 */
#define ROTATE_POS(in,out,t)	out.q[N]=_IQmpy(in.q[N],_IQcos(t))-_IQmpy(in.d[N],_IQsin(t));out.d[N]=_IQmpy(in.q[N],_IQsin(t))+_IQmpy(in.d[N],_IQcos(t));

/*! \def ROTATE_NEG(in,out,t)
 * \ingroup ROTATE_QDDATA 
 * define negative rotation of qddata (rotate to stationary coordinates out=in*exp(-j*t))
 * \details
 * \a in data in the original reference frame.
 * 
 * \a out data in the rotated reference frame.
 * 
 * \a t angle of rotation.
 */
#define ROTATE_NEG(in,out,t)	out.q[N]=_IQmpy(in.q[N],_IQcos(t))+_IQmpy(in.d[N],_IQsin(t));out.d[N]=-_IQmpy(in.q[N],_IQsin(t))+_IQmpy(in.d[N],_IQcos(t));


/// \defgroup ROTATE_SQDDATA define positive & negative rotation of sqddata

/*! \def SROTATE_POS(in,out,t) 
 * \ingroup ROTATE_SQDDATA 
 * define positive rotation of sqddata (rotate to synchronous coordinates out=in*exp(j*t))
 *
 * \a in data in the original reference frame.
 * 
 * \a out data in the rotated reference frame.
 * 
 * \a t angle of rotation.
 */
#define SROTATE_POS(in,out,t)	out.q=_IQmpy(in.q,_IQcos(t))-_IQmpy(in.d,_IQsin(t));out.d=_IQmpy(in.q,_IQsin(t))+_IQmpy(in.d,_IQcos(t));

/*! \def SROTATE_NEG(in,out,t)
 * \ingroup ROTATE_SQDDATA
 * define negative rotation of sqddata (rotate to synchronous coordinates out=in*exp(j*t))
 *
 * \a in data in the original reference frame.
 * 
 * \a out data in the rotated reference frame.
 * 
 * \a t angle of rotation.
 */
#define SROTATE_NEG(in,out,t)	out.q=_IQmpy(in.q,_IQcos(t))+_IQmpy(in.d,_IQsin(t));out.d=-_IQmpy(in.q,_IQsin(t))+_IQmpy(in.d,_IQcos(t));


/// \defgroup ROTATE_ESC define positive & negative rotation of scalar input

/*!
 * \def ROTATE_POS_ESC(in,out,t)
 * \ingroup ROTATE_ESC
 * define positive rotation of scaler input (rotate to synchronous coordinates out=in*exp(j*t))
 *
 * \a in (scalar) data in the original reference frame.
 * 
 * \a out (qddata) data in the rotated reference frame.
 * 
 * \a t angle of rotation.
 */
#define ROTATE_POS_ESC(in,out,t)	out.q[N]=_IQmpy(in,_IQcos(t));out.d[N]=_IQmpy(in,_IQsin(t));

/*!
 * \def ROTATE_NEG_ESC(in,out,t)
 * \ingroup ROTATE_ESC
 * define negative rotation of sqddata (rotate to synchronous coordinates out=in*exp(j*t))
 *
 * \a in data in the original reference frame.
 * 
 * \a out data in the rotated reference frame.
 * 
 * \a t angle of rotation.
 */
#define ROTATE_NEG_ESC(in,out,t)	out.q[N]=_IQmpy(in,_IQcos(t));out.d[N]=-_IQmpy(in,_IQsin(t));

/*!
 * \def SROTATE_POS_ESC(in,out,t)
 * \ingroup ROTATE_ESC
 */
#define SROTATE_POS_ESC(in,out,t)	out.q=_IQmpy(in,_IQcos(t));out.d=_IQmpy(in,_IQsin(t));

/*!
 * \def SROTATE_NEG_ESC(in,out,t)
 * \ingroup ROTATE_ESC
 */
#define SROTATE_NEG_ESC(in,out,t)	out.q=_IQmpy(in,_IQcos(t));out.d=-_IQmpy(in,_IQsin(t));


/// \defgroup PI_REG PI regulator G(s)=Kp*(s+Ki)/s. Tustin discretization

/*!
 * \def PI_REG(in,out,Kp,Ki)
 * \ingroup PI_REG
 */
#define PI_REG(in,out,Kp,Ki)		out[N]=out[N1]+_IQmpy(Kp,_IQmpy(in[N],(_IQ(1)+_IQmpy(DTDIV2,Ki))) + _IQmpy(in[N1],(_IQmpy(DTDIV2,Ki)-_IQ(1))))

/*!
 * \def PI_QD_REG(in,out,Kp,Ki)
 * \ingroup PI_REG
 */
#define PI_QD_REG(in,out,Kp,Ki)	PI_REG(in.q,out.q,Kp,Ki);	PI_REG(in.d,out.d,Kp,Ki)

/*!
 * \def PI_QD_REG_NOSYM(in,out,Kp_q,Ki_q,Kp_d,Ki_d)
 * \ingroup PI_REG
 */
#define PI_QD_REG_NOSYM(in,out,Kp_q,Ki_q,Kp_d,Ki_d)	PI_REG(in.q,out.q,Kp_q,Ki_q);	PI_REG(in.d,out.d,Kp_d,Ki_d)

/*!
 * \def PI_REG_DT(in,out,Kp,Ki,dTDiv2)
 * \ingroup PI_REG
 */
#define PI_REG_DT(in,out,Kp,Ki,dTDiv2)		out[N]=out[N1]+_IQmpy(Kp,_IQmpy(in[N],(_IQ(1)+_IQmpy(dTDiv2,Ki))) + _IQmpy(in[N1],(_IQmpy(dTDiv2,Ki)-_IQ(1))))

/// \defgroup PID_REG PID regulator G(s) = Kp+Ki/s+Kd*s. Tustin discretization

/*!
 * \def PID_COEFFS_LPF(kp,ki,kd,lpf,b0,b1,b2,a1,a2)
 * \ingroup PID_REG
 * \details 
 * numerator
	b0 = (Kp*2*T+Ki*T^2+4*Kd)/(4+2*lpf*T);
	b1 = (8*Ki*T^2-8*Kd)/(4+2*lpf*T);
	b2 = (-Kp*2*T+Ki*T^2+4*Kd)/(4+2*lpf*T);
 * denominator
    a0 = 1;
    a1 = (-8)/(4+2*lpf*T);
    a2 = (4-2*lpf*T)/(4+2*lpf*T);
 * difference equation:
    y[k] = -a1*y[k-1]-a2*y[k-2]+b0*u[k]+b1*u[k-1]+b2*u[k-2]
 */
#define PID_COEFFS_LPF(kp,ki,kd,lpf,b0,b1,b2,a1,a2)	{\
												b0 = _IQ(lpf*(kp*2*DTF+ki*DTF*DTF+4*kd)/(4+2*lpf*DTF));\
												b1 = _IQ(lpf*(8*ki*DTF*DTF-8*kd)/(4+2*lpf*DTF));\
												b2 = _IQ(lpf*(-kp*2*DTF+ki*DTF*DTF+4*kd)/(4+2*lpf*DTF));\
												a1 = _IQ((-8)/(4+2*lpf*DTF));\
												a2 = _IQ((4-2*lpf*DTF)/(4+2*lpf*DTF));\
												}

/*!
 * \def PID_REG_LPF(in,out,b0,b1,b2,a1,a2)
 * \ingroup PID_REG
 */
#define PID_REG_LPF(in,out,b0,b1,b2,a1,a2)	out[N] = - _IQmpy(a1,out[N1]) - _IQmpy(a2,out[N2]) + _IQmpy(b0,in[N]) + _IQmpy(b1,in[N1]) + _IQmpy(b2,in[N2])

/*!
 * \def PID_QD_REG_LPF(in,out,b0,b1,b2,a1,a2)
 * \ingroup PID_REG
 */
#define PID_QD_REG_LPF(in,out,b0,b1,b2,a1,a2)	PID_REG(in.q,out.q,b0,b1,b2,a1,a2);	PID_REG(in.d,out.d,b0,b1,b2,a1,a2)

/*!
 * \def PID_COEFFS(kp,ki,kd,b0,b1,b2)
 * \ingroup PID_REG
 * PID Regulator (no lpf)
 */
#define PID_COEFFS(kp,ki,kd,b0,b1,b2)	{\
										b0 = _IQmpy(ki,DTDIV2)+_IQdiv(kd,DTDIV2)+kp;\
										b1 = _IQmpy(ki,DT)    -_IQdiv(kd,DTDIV4);\
										b2 = _IQmpy(ki,DTDIV2)+_IQdiv(kd,DTDIV2)-kp;\
										}
										
										
/*!
 * \def PID_REG(in,out,k0,k1,k2)
 * \ingroup PID_REG
 */
#define PID_REG(in,out,k0,k1,k2)			out[N] = _IQmpy(k0,in[N]) + _IQmpy(k1,in[N1]) + _IQmpy(k2,in[N2]) + out[N2]

/*!
 * \def INV_PID_REG(in,out,k0,k1,k2)
 * \ingroup PID_REG
 */
#define INV_PID_REG(in,out,k0,k1,k2)		in[N]  = _IQdiv((out[N]-out[N2]- _IQmpy(k1,in[N1])- _IQmpy(k2,in[N2])),k0)

/*!
 * \def PID_QD_REG(in,out,k0,k1,k2)
 * \ingroup PID_REG
 */
#define PID_QD_REG(in,out,k0,k1,k2)			PID_REG(in.d,out.d,k0,k1,k2);		PID_REG(in.q,out.q,k0,k1,k2)

/*!
 * \def INV_PID_QD_REG(in,out,k0,k1,k2)
 * \ingroup PID_REG
 */
#define INV_PID_QD_REG(in,out,k0,k1,k2)		INV_PID_REG(in.d,out.d,k0,k1,k2);	INV_PID_REG(in.q,out.q,k0,k1,k2)


/*!
 * \def INV_PI_REG(in,out,Kp,Ki)
 * \ingroup PI_REG
 * inverse PI --> calculate input for a given output to implement realizable references
 */
#define INV_PI_REG(in,out,Kp,Ki)	{\
									in[N] = _IQdiv(out[N] - out[N1] -\
									_IQmpy(Kp,_IQmpy(_IQmpy(DTDIV2,Ki)-_IQ(1), in[N1])),\
									_IQmpy(Kp, _IQ(1)+_IQmpy(DTDIV2,Ki) ) );\
									}
									
/*!
 * \def INV_PI_QD_REG(in,out,Kp,Ki)
 * \ingroup PI_REG
 * inverse PI --> calculate input for a given output to implement realizable references
 */									
#define INV_PI_QD_REG(in,out,Kp,Ki)	INV_PI_REG(in.q,out.q,Kp,Ki); INV_PI_REG(in.d,out.d,Kp,Ki)

/*!
 * \def INV_PI_QD_REG_NOSYM(in,out,Kp_q,Ki_q,Kp_d,Ki_d)
 * \ingroup PI_REG
 * inverse PI --> calculate input for a given output to implement realizable references
 */
#define INV_PI_QD_REG_NOSYM(in,out,Kp_q,Ki_q,Kp_d,Ki_d)	INV_PI_REG(in.q,out.q,Kp_q,Ki_q); INV_PI_REG(in.d,out.d,Kp_d,Ki_d)
	

/// \deprecated PI_REG_WR(in,out,Kp,Ki)
#define PI_REG_WR(in,out,Kp,Ki)		out[N]=out[N1]+_IQmpy(Kp,_IQmpy(in[N],(_IQ(1)+_IQmpy(DTWDIV2,Ki))) + _IQmpy(in[N1],(_IQmpy(DTWDIV2,Ki)-_IQ(1))))

/// \defgroup INT_D 1/s. Tustin discretization

/*!
 * \def INT(in,out)
 * \ingroup INT_D
 * Discrete integral for data variables
 */
#define INT(in,out)			out[N]=out[N1]+_IQmpy(DTDIV2,(in[N]+in[N1]))

/*!
 * \def INT_QD(in, out)
 * \ingroup INT_D
 * Discrete integral for qddata variables
 */
#define INT_QD(in, out)		INT(in.q,out.q); INT(in.d,out.d);

/*!
 * \def INT_DT(in,out,dTDiv2)
 * \ingroup INT_D
 * Discrete integral for data variables (external T/2)
 */
#define INT_DT(in,out,dTDiv2)		out[N]=out[N1]+_IQmpy(dTDiv2,(in[N]+in[N1]))

/*!
 * \def INT_QD_DT(in, out,dTDiv2)
 * \ingroup INT_D
 * Discrete integral for qddata variables (external T/2)
 */
#define INT_QD_DT(in, out,dTDiv2)	INT_DT(in.q,out.q,dTDiv2); INT_DT(in.d,out.d,dTDiv2);


//-----------------------------------------------------------------------------
/// \defgroup	GENERAL_SLPF general SLPF (stationary low-pass filters), dqdata
//-----------------------------------------------------------------------------

/*!
 * \def FILTER_LPF(in,out,B0_LPF,A1_LPF)
 * \ingroup GENERAL_SLPF
 * low pass filter data
 */
#define FILTER_LPF(in,out,B0_LPF,A1_LPF)	out[N]=_IQmpy(B0_LPF,(in[N]+in[N1]))+_IQmpy(A1_LPF,out[N1])

/*!
 * \def FILTER_Q_LPF(in,out,B0_LPF,A1_LPF)
 * \ingroup GENERAL_SLPF
 * low pass filter qddata (q-axis component)
 */
#define FILTER_Q_LPF(in,out,B0_LPF,A1_LPF)	out.q[N]=_IQmpy(B0_LPF,(in.q[N]+in.q[N1]))+_IQmpy(A1_LPF,out.q[N1])

/*!
 * \def FILTER_D_LPF(in,out,B0_LPF,A1_LPF)
 * \ingroup GENERAL_SLPF
 * low pass filter qddata (d-axis component)
 */
#define FILTER_D_LPF(in,out,B0_LPF,A1_LPF)	out.d[N]=_IQmpy(B0_LPF,(in.d[N]+in.d[N1]))+_IQmpy(A1_LPF,out.d[N1])

/*!
 * \def FILTER_QD_LPF(in,out,B0_LPF,A1_LPF)
 * \ingroup GENERAL_SLPF
 * low pass filter qddata (q-axis,d-axis components)
 */
#define FILTER_QD_LPF(in,out,B0_LPF,A1_LPF)	FILTER_Q_LPF(in,out,B0_LPF,A1_LPF); FILTER_D_LPF(in,out,B0_LPF,A1_LPF)

//-----------------------------------------------------------------------------
/// \defgroup	GENERAL_SHPF general SHPF (stationary high-pass filters), dqdata
//-----------------------------------------------------------------------------

/*!
 * \def FILTER_HPF(in,out,B0_LPF,A1_LPF)
 * \ingroup GENERAL_SHPF
 * high pass filter data
 */
#define FILTER_HPF(in,out,B0_HPF,A1_HPF)	out[N]=_IQmpy(B0_HPF,(in[N]-in[N1]))+_IQmpy(A1_HPF,out[N1])

/*!
 * \def FILTER_Q_HPF(in,out,B0_HPF,A1_HPF)
 * \ingroup GENERAL_SHPF
 * high pass filter qddata (q-axis component)
 */
#define FILTER_Q_HPF(in,out,B0_HPF,A1_HPF)	out.q[N]=_IQmpy(B0_HPF,(in.q[N]-in.q[N1]))+_IQmpy(A1_HPF,out.q[N1])

/*!
 * \def FILTER_D_HPF(in,out,B0_HPF,A1_HPF)
 * \ingroup GENERAL_SHPF
 * high pass filter qddata (d-axis component)
 */
#define FILTER_D_HPF(in,out,B0_HPF,A1_HPF)	out.d[N]=_IQmpy(B0_HPF,(in.d[N]-in.d[N1]))+_IQmpy(A1_HPF,out.d[N1])

/*!
 * \def FILTER_QD_HPF(in,out,B0_HPF,A1_HPF)
 * \ingroup GENERAL_SHPF
 * high pass filter qddata (q-axis, d-axis components)
 */
#define FILTER_QD_HPF(in,out,B0_HPF,A1_HPF)     FILTER_Q_HPF(in,out,B0_HPF,A1_HPF); FILTER_D_HPF(in,out,B0_HPF,A1_HPF)

//-----------------------------------------------------------------------------
/// \defgroup	GENERAL_FILTERING General Filtering (FIR, IIR) functions.
//-----------------------------------------------------------------------------

/*!
* General FIR filter: out[0] = b[0]*in[0] + b[1]*in[1] + ... + b[n]*in[n]
* \ingroup GENERAL_FILTERING
* \param in input vector. Vector MUST have size = n
* \param b input coefficients vector. Vector MUST have size = n
* \param n size of the filter
*/
static inline _iq FirFilter(volatile _iq* in, _iq* b, unsigned int n)
{
        unsigned int cc;
	_iq out = 0;
        for(cc=0;cc<n;cc++)
                out += _IQmpy(in[cc],b[cc]);
	return out;
}

//-----------------------------------------------------------------------------

/*!
* General FIR filter: out[0] = b[0]*in[0] + b[1]*in[1] + ... + b[n]*in[n]
* \ingroup GENERAL_FILTERING
* \param in input vector. Vector MUST have size = n
* \param b input coefficients vector. Vector MUST have size = n
* \param n size of the filter
* TODO: test
*/
static inline _iq FirFilterV2(volatile _iq* in, _iq* b, unsigned int n)
{
        unsigned int cc;
        register _iq temp;
        temp = _IQmpy(*in++,*b++);
        for(cc=1;cc<n;cc++)
               temp += _IQmpy(*in++,*b++);
        return temp;
}

//-----------------------------------------------------------------------------

/*!
* General IIR filter: out[0] = b[0]*in[0] + ... + b[nb]*in[nb] - (a[0]*out[0] + ... + a[na-1]*out[na])
* \ingroup GENERAL_FILTERING
* \param in input vector. Vector MUST have size = nb
* \param out output vector. After execution, out[0] contains the updated value
* \param b input coefficients vector. Vector MUST have size = nb
* \param nb size of input
* \param a output coefficients vector. Vector MUST have size = na
* \param na size of output
*/
static inline _iq IIRFilter(volatile _iq* in, volatile _iq* out, 
	_iq* b, unsigned int nb, _iq* a, unsigned int na)
{
	return FirFilter(in,b,nb) - FirFilter(out,a,na);
}

//-----------------------------------------------------------------------------


 /*!
 * Shift vector right. After execution: in[n-1]=in[n-2],...,in[1]=in[0]
 * \ingroup GENERAL_FILTERING
 * \param in input vector. Vector MUST have size = n
 * \param n size of the vector
 */
static inline void ShiftRight(volatile _iq* in, unsigned int n)
{
        unsigned int cc;
        for(cc=n-1;cc>0;cc--)
                in[cc] = in[cc-1];
}

//-----------------------------------------------------------------------------

 /*!
 * Shift vector right. After execution: in[n-1]=in[n-2],...,in[1]=in[0]
 * \ingroup GENERAL_FILTERING
 * \param in input vector. Vector MUST have size = n
 * \param n size of the vector
 * TODO: test
 */
static inline void ShiftRightV2(volatile _iq* in, unsigned int n)
{
        unsigned int cc;
        volatile _iq* yend = &(in[n-1]);
        for(cc=0;cc<n;cc++)
            *yend = *(--yend);
}

//-----------------------------------------------------------------------------

/*!
* Initializes vector v components to be zero
* \ingroup GENERAL_FILTERING
* \param v input vector. Vector MUST have size = n
* \param n size of the vector
*/
static inline void Zeros(volatile _iq* v, unsigned int n)
{
        unsigned int cc;
        for(cc=0;cc<n;cc++)
                v[cc] = 0;
}

//-----------------------------------------------------------------------------

/*!
* Initializes Moving Average Coefficients
* \ingroup GENERAL_FILTERING
* \param b coefficients vector. Vector MUST have size = n
* \param n size of the vector
*/
static inline void MAvrgCoeffs(_iq* b, unsigned int n)
{
        unsigned int cc;
	_iq b0 = _IQdiv(_IQ(1.0),_IQ(n));
        for(cc=0;cc<n;cc++)
                b[cc] = b0;
}

//-----------------------------------------------------------------------------


#ifdef Q_AXIS_REAL
        // 2->3 transform
	#define DQ2PH_QDDATA(fqd,fa,fb,fc)		fa=fqd.q[N];fb=_IQmpy(_IQ(-0.5),fqd.q[N])-_IQmpy(SQRT3DIV2,fqd.d[N]);fc=-(fa+fb)
	#define DQ2PH(fqd,fa,fb,fc)				fa=fqd.q;fb=_IQmpy(_IQ(-0.5),fqd.q)-_IQmpy(SQRT3DIV2,fqd.d);fc=-(fa+fb)
	// 3 ->2 transform
	#define PH2QD(fa,fb,fqd)				fqd.q=fa;fqd.d=-(_IQmpy(SQRT3DIV3,fa)+_IQmpy(TWOSQRT3DIV3,fb));
	#define PH2QD_QDDATA(fa,fb,fqd)			fqd.q[N]=fa;fqd.d[N]=-(_IQmpy(SQRT3DIV3,fa)+_IQmpy(TWOSQRT3DIV3,fb));
	// cross-product
	#define CROSS(in,saliency,out)			out=_IQmpy(-in.d,_IQcos(saliency))-_IQmpy(in.q,_IQsin(saliency))
	#define CROSS_QDDATA(in,saliency,out)	out=_IQmpy(-in.d[N],_IQcos(saliency))-_IQmpy(in.q[N],_IQsin(saliency))
	// build a rotating carrier vector
	#define SCALAR2SYNCVEC(in,out,t)		out.q[N]=_IQmpy(in , _IQcos(t));out.d[N]=-_IQmpy(in , _IQsin(t));
	#define SSCALAR2SYNCVEC(in,out,t)		out.q=_IQmpy(in , _IQcos(t));out.d=-_IQmpy(in , _IQsin(t));
	// build a pulsating carrier signal
	#define SCALAR2PULSATING(in,out,t)		out.q[N]=_IQ(0);	out.d[N]=-_IQmpy(in , _IQcos(t));
	#define SSCALAR2PULSATING(in,out,t)		out.q=_IQ(0);		out.d=-_IQmpy(in , _IQcos(t));

#else 
#ifdef D_AXIS_REAL
	/// \defgroup 2to3TRF 2->3 transform
	
	/*!
	 * \def DQ2PH_QDDATA(fqd,fa,fb,fc)
	 * \ingroup 2to3TRF
	 * 
	 */
	#define DQ2PH_QDDATA(fqd,fa,fb,fc)		fa=fqd.d[N];fb=_IQmpy(_IQ(-0.5),fqd.d[N])+_IQmpy(SQRT3DIV2,fqd.q[N]);fc=-(fa+fb)

	/*!
	 * \def DQ2PH(fqd,fa,fb,fc)
	 * \ingroup 2to3TRF
	 * 
	 */
	#define DQ2PH(fqd,fa,fb,fc)				fa=fqd.d;fb=_IQmpy(_IQ(-0.5),fqd.d)+_IQmpy(SQRT3DIV2,fqd.q);fc=-(fa+fb)

	/// \defgroup 3to2TRF 3->2 transform
	
	/*!
	 * \def PH2QD(fa,fb,fqd)
	 * \ingroup 3to2TRF
	 * 
	 */	
	#define PH2QD(fa,fb,fqd)				fqd.d=fa;fqd.q=(_IQmpy(SQRT3DIV3,fa)+_IQmpy(TWOSQRT3DIV3,fb));
	
	/*!
	 * \def PH2QD_QDDATA(fa,fb,fqd)
	 * \ingroup 3to2TRF
	 * 
	 */
	#define PH2QD_QDDATA(fa,fb,fqd)			fqd.d[N]=fa;fqd.q[N]=(_IQmpy(SQRT3DIV3,fa)+_IQmpy(TWOSQRT3DIV3,fb));
	
	/// \defgroup CROSS_PRODUCT cross product

	/*!
	 * \def CROSS(in,saliency,out)
	 * \ingroup CROSS_PRODUCT
	 * 
	 */
	#define CROSS(in,saliency,out)			out=_IQmpy(in.d,_IQsin(saliency))-_IQmpy(in.q,_IQcos(saliency))

	/*!
	 * \def CROSS_QDDATA(in,saliency,out)
	 * \ingroup CROSS_PRODUCT
	 * 
	 */
	#define CROSS_QDDATA(in,saliency,out)	out=_IQmpy(in.d[N],_IQsin(saliency))-_IQmpy(in.q[N],_IQcos(saliency))
	
	/// \defgroup SCALAR2SYNCVEC build a rotating carrier vector
	
	/*!
	 * \def SCALAR2SYNCVEC(in,out,t)
	 * \ingroup SCALAR2SYNCVEC
	 * 
	 */
	#define SCALAR2SYNCVEC(in,out,t)		out.d[N]=_IQmpy(in , _IQcos(t));out.q[N]=_IQmpy(in , _IQsin(t));

	/*!
	 * \def SSCALAR2SYNCVEC(in,out,t)
	 * \ingroup SCALAR2SYNCVEC
	 * 
	 */
	#define SSCALAR2SYNCVEC(in,out,t)		out.d=_IQmpy(in , _IQcos(t));out.q=_IQmpy(in , _IQsin(t));
	
	/// \defgroup SCALAR2PULSATING build a pulsating carrier signal

	/*!
	 * \def SCALAR2PULSATING(in,out,t)
	 * \ingroup SCALAR2PULSATING
	 * 
	 */
	#define SCALAR2PULSATING(in,out,t)		out.d[N]=_IQmpy(in , _IQcos(t));out.q[N]=_IQ(0);	

	/*!
	 * \def SSCALAR2PULSATING(in,out,t)
	 * \ingroup SCALAR2PULSATING
	 * 
	 */
	#define SSCALAR2PULSATING(in,out,t)		out.d=_IQmpy(in , _IQcos(t));out.q=_IQ(0);	
#else
	#error Q_AXIS_REAL or D_AXIS_REAL must be defined to use this header
#endif
#endif


#endif
//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
