/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	QDOperations.h
 Project	:  	TIMotorLIB
 
 TI Doc:	na
 \author  	Pablo Garcia (pgarcia@isa.uniovi.es)
 \brief 	Complex number operations in \a dq format
 \date 		2007
******************************************************************************/

#ifndef QDOPERATIONS_H
#define QDOPERATIONS_H

#include "IQConstants.h"

/// \defgroup ABS_QDDATA absolute value for QD data variables ( $sqrt(in.d^2+in.q^2)$ )

/*!
 * \def MAG_QD(in)
 * \ingroup ABS_QDDATA
 * 
 */
#define MAG_QD(in) 					(_IQmag(in.q,in.d))

/*!
 * \def MAG_QD_NDATA(in,n)
 * \ingroup ABS_QDDATA
 * 
 */
#define MAG_QD_NDATA(in,n) 			(_IQmag(in.q[n],in.d[n]))

/*!
 * \def MAG_QD_DATA(in,out)
 * \ingroup ABS_QDDATA
 * 
 */
#define MAG_QD_DATA(in,out) 		{\
									out[N] = MAG_QD_NDATA(in,N);\
									out[N1]= MAG_QD_NDATA(in,N1);\
									}

/// \defgroup SQR_QDDATA square value for QD data variables ( $in.d^2+in.q^2$ )

/*!
 * \def SQR_QD(in)
 * \ingroup SQR_QDDATA
 * 
 */
#define SQR_QD(in) 					(_IQmpy(in.q,in.q)+_IQmpy(in.d,in.d))

/*!
 * \def SQR_QD_NDATA(in,n)
 * \ingroup SQR_QDDATA
 * 
 */
#define SQR_QD_NDATA(in,n) 			(_IQmpy(in.q[n],in.q[n])+_IQmpy(in.d[n],in.d[n]))

/*!
 * \def SQR_QD_DATA(in,out)
 * \ingroup SQR_QDDATA
 * 
 */
#define SQR_QD_DATA(in,out) 		{\
									out[N]=SQR_QD_NDATA(in,N);\
									out[N1]=SQR_QD_NDATA(in,N1);\
									}


/// \defgroup SUM_QDDATA sum QD data ($out.d = s1.d+s2.d$, $out.q = s1.q+s2.q$)

/*!
 * \def SUM_QD(s1,s2,out)
 * \ingroup SUM_QDDATA
 * 
 */
#define SUM_QD(s1,s2,out) 			{out.q=s1.q+s2.q;out.d=s1.d+s2.d;}

/*!
 * \def SUM_QD_NDATA(s1,s2,out,n)
 * \ingroup SUM_QDDATA
 * 
 */
#define SUM_QD_NDATA(s1,s2,out,n)	{out.q[n]=s1.q[n]+s2.q[n];out.d[n]=s1.d[n]+s2.d[n];}

/*!
 * \def SUM_QD_DATA(s1,s2,out)
 * \ingroup SUM_QDDATA
 * 
 */
#define SUM_QD_DATA(s1,s2,out) 		{\
									SUM_QD_NDATA(s1,s2,out,N);\
									SUM_QD_NDATA(s1,s2,out,N1);\
									}

/// \defgroup DIFF_QDDATA sum QD data ($out.d = s1.d-s2.d$, $out.q = s1.q-s2.q$)

/*!
 * \def DIFF_QD(s1,s2,out)
 * \ingroup DIFF_QDDATA
 * 
 */
#define DIFF_QD(s1,s2,out) 			{out.q=s1.q-s2.q;out.d=s1.d-s2.d;}

/*!
 * \def DIFF_QD_NDATA(s1,s2,out,n)
 * \ingroup DIFF_QDDATA
 * 
 */
#define DIFF_QD_NDATA(s1,s2,out,n)	{out.q[n]=s1.q[n]-s2.q[n];out.d[n]=s1.d[n]-s2.d[n];}

/*!
 * \def DIFF_QD_DATA(s1,s2,out)
 * \ingroup DIFF_QDDATA
 * 
 */
#define DIFF_QD_DATA(s1,s2,out) 	{\
									DIFF_QD_NDATA(s1,s2,out,N);\
									DIFF_QD_NDATA(s1,s2,out,N1);\
									}

/*! \def INV_QD(in,out)
 * Calculate the inverse of \a qd variable
 * 
 */
#define INV_QD(in,out)              {out.q=_IQdiv(_IQ(1),in.q);out.d=_IQdiv(_IQ(1),in.d);}

/*! \def MULT_REAL(dq,sc,out)
 * Multiplication of \a qd variable and \a sc real value
 */
#define MULT_REAL(dq,sc,out)		{out.q=_IQmpy(dq.q,sc);out.d=_IQmpy(dq.d,sc);}

/*! \def _NEG_QD(in)
 * negate variable
 */
#define _NEG_QD(in)					{in.q=-in.q;in.d=-in.d;}
#define NEG_QD(in,out)				{out.q=-in.q;out.d=-in.d;}

/*! \def ASSIGN_QD(in,out) 
 * assign \a in variable to \a out variable
 */
#define ASSIGN_QD(in,out)			{out.q=in.q;out.d=in.d;}

/*! \def SIGN(in) 
 * obtain sign of _iq variable. It returns 1 if positive and -1 if negative
 * */
#define SIGN(in)					(1+((in & (long)0x80000000)>>30))//1-2*[1,0]

#ifdef QUADRATURE_AXIS_REAL
	// make QD
	#define MAKE_QD(real,imag,out)	out.q=real; out.d=-imag
	// real & imaginary
    #define REAL_QD(in)             (in.q)
    #define IMAG_QD(in)             (-in.d)
    #define REAL_QD_DATA(in)        (in.q[N])
    #define IMAG_QD_DATA(in)        (-in.d[N])
	// conjugate
    #define _CONJ_QD(in)         	in.d=-in.d
    #define CONJ_QD(in,out)         {out.q=in.q;out.d=-in.d;}
    #define CONJ_QD_NDATA(in,out,n) {out.q[n]=in.q[n];out.d[n]=-in.d[n];}
	#define CONJ_QD_DATA(in,out)	{\
                                    CONJ_QD_NDATA(in,out,N);\
                                    CONJ_QD_NDATA(in,out,N1);\
                                    }
	// phase QD data
    #define _PHASE_QD(in)           (_IQatan2(-in.d,in.q))
    #define PHASE_QD(in,out) 		out=_PHASE_QD(in);
    #define PHASE_QD_NDATA(in,n)	(_IQatan2(-in.d[n],in.q[n]))
    #define PHASE_QD_DATA(in,out)   {\
                                    out[N]=PHASE_QD_NDATA(in,N);\
                                    out[N1]=PHASE_QD_NDATA(in,N1);\
                                    }
	// mult QD data
    #define TIMES_Q(s1,s2)          (_IQmpy(s1.q,s2.q)-_IQmpy(s1.d,s2.d))
    #define TIMES_D(s1,s2)          (_IQmpy(s1.q,s2.d)+_IQmpy(s1.d,s2.q))
    #define TIMES_QD(s1,s2,out)     {out.q=TIMES_Q(s1,s2);out.d=TIMES_D(s1,s2);}
    #define TIMES_Q_DATA(s1,s2,n)	(_IQmpy(s1.q[n],s2.q[n])-_IQmpy(s1.d[n],s2.d[n]))
    #define TIMES_D_DATA(s1,s2,n)	(_IQmpy(s1.q[n],s2.d[n])+_IQmpy(s1.d[n],s2.q[n]))
    #define TIMES_QD_NDATA(s1,s2,out,n){\
                                    out.q[n]=TIMES_Q_DATA(s1,s2,n);\
                                    out.d[n]=TIMES_D_DATA(s1,s2,n);\
                                    }
    #define TIMES_QD_DATA(s1,s2,out){\
									TIMES_QD_NDATA(s1,s2,out,N);\
									TIMES_QD_NDATA(s1,s2,out,N1);\
									}                                    }

	// sqrt QD data
    #define SQRT_QD(in,out)         {\
                                    _iq mag = _IQsqrt(MAG_QD(in));\
                                    _iq phase = _PHASE_QD(in)/2;\
                                    REAL_QD(out)=_IQmpy(mag,_IQcos(phase));\
                                    IMAG_QD(out)=_IQmpy(mag,_IQsin(phase));\
                                    }

    // pow QD data
    #define POW_QD(in,out,pow)     {\
                                    _iq mag = _IQsqrt(MAG_QD(in));\
                                    _iq phase = _IQmpy(_PHASE_QD(in),_IQ(pow));\
                                    REAL_QD(out)=_IQmpy(mag,_IQcos(phase));\
                                    IMAG_QD(out)=_IQmpy(mag,_IQsin(phase));\
                                    }
#else
	/*! \def MAKE_QD(real,imag,out)
	 * make qd variable from \a real and \a imag components
	 */
	#define MAKE_QD(real,imag,out)	out.d=real; out.q=imag
	/*! \defgroup REAL_IMAG real & imaginary macros
	 * real & imaginary macros
	 * @{
	 */
    #define REAL_QD(in)             (in.d)
    #define IMAG_QD(in)             (in.q)
    #define REAL_QD_DATA(in)        (in.d[N])
    #define IMAG_QD_DATA(in)        (in.q[N])
	/*! @} */
	
	/*! \defgroup MOD_ARG mod and argument macros
	 * mod and argument macros
	 * @{
	 */
    #define MODARG2QD(mod,arg,out)	{\
                                    out.d = _IQmpy(mod,_IQcos(arg));\
                                    out.q = _IQmpy(mod,_IQsin(arg));\
                                    }
	#define MODARG2QD_NDATA(mod,arg,out,n)	{\
										 	out.d[n] = _IQmpy(mod,_IQcos(arg));\
										 	out.q[n] = _IQmpy(mod,_IQsin(arg));\
											}
	#define MODARG2QD_DATA(mod,arg,out)		{\
										 	MODARG2QD_NDATA(mod,arg,out,N);\
										 	MODARG2QD_NDATA(mod,arg,out,N1);\
											}
	/*! @} */
	
	/*! \defgroup conjugate Conjugate macros
	 * @{
	 */
    #define _CONJ_QD(in)         	in.q=-in.q
    #define CONJ_QD(in,out)         {out.d=in.d;out.q=-in.q;}
    #define CONJ_QD_NDATA(in,out,n) {out.d[n]=in.d[n];out.q[n]=-in.q[n];}
    #define CONJ_QD_DATA(in,out)	{\
                                    CONJ_QD_NDATA(in,out,N);\
                                    CONJ_QD_NDATA(in,out,N1);\
                                    }
	/*! @} */
	
	/*! \defgroup PHASE_QD phase QD data macros
	 * @{
	 */
    #define _PHASE_QD(in)           (_IQatan2(in.q,in.d))
    #define PHASE_QD(in,out) out=   _PHASE_QD(in);
    #define PHASE_QD_NDATA(in,n)	(_IQatan2(in.q[n],in.d[n]))
    #define PHASE_QD_DATA(in,out)   {\
                                    out[N]=PHASE_QD_NDATA(in,N);\
                                    out[N1]=PHASE_QD_NDATA(in,N1);\
                                    }
	/*! @} */
	
	/*! \defgroup MULT_QD_DATA mult QD data variables
	 * @{
	 */
    #define TIMES_D(s1,s2)          (_IQmpy(s1.d,s2.d)-_IQmpy(s1.q,s2.q))
    #define TIMES_Q(s1,s2)          (_IQmpy(s1.q,s2.d)+_IQmpy(s1.d,s2.q))
    #define TIMES_QD(s1,s2,out)     out.d=TIMES_D(s1,s2);out.q=TIMES_Q(s1,s2)
    #define TIMES_D_DATA(s1,s2,n)	(_IQmpy(s1.d[n],s2.d[n])-_IQmpy(s1.q[n],s2.q[n]))
    #define TIMES_Q_DATA(s1,s2,n)	(_IQmpy(s1.q[n],s2.d[n])+_IQmpy(s1.d[n],s2.q[n]))
    #define TIMES_QD_NDATA(s1,s2,out,n){\
                                    out.d[n]=TIMES_D_DATA(s1,s2,n);\
                                    out.q[n]=TIMES_Q_DATA(s1,s2,n);\
                                    }
    #define TIMES_QD_DATA(s1,s2,out){\
									TIMES_QD_NDATA(s1,s2,out,N);\
									TIMES_QD_NDATA(s1,s2,out,N1);\
                                    }
	/*#define TIMES_QD_DATA(s1,s2,out) {\
												out.d[N]=_IQmpy(s1.d[N],s2.d[N])-_IQmpy(s1.q[N],s2.q[N]);\
												out.q[N]=_IQmpy(s1.q[N],s2.d[N])+_IQmpy(s1.d[N],s2.q[N]);\
												out.d[N1]=_IQmpy(s1.d[N1],s2.d[N1])-_IQmpy(s1.q[N1],s2.q[N1]);\
												out.q[N1]=_IQmpy(s1.q[N1],s2.d[N1])+_IQmpy(s1.d[N1],s2.q[N1]);\
												}*/
	/*! @} */
	
	/*! \def SQRT_QD(in,out)
	 * sqrt \a qddata variable
	 */
    #define SQRT_QD(in,out)     {\
                                _iq mag = _IQsqrt(MAG_QD(in));\
                                _iq phase = _PHASE_QD(in)/2;\
                                REAL_QD(out)=_IQmpy(mag,_IQcos(phase));\
                                IMAG_QD(out)=_IQmpy(mag,_IQsin(phase));\
								}

#endif

/*! \defgroup DIV_QD_DATA division of QD data
 * @{
 */
#define DIV_QD(s1,s2,temp,out) 	{\
                                CONJ_QD(s2,temp);\
                                out.q=_IQdiv(TIMES_Q(s1,temp),SQR_QD(s2));\
                                out.d=_IQdiv(TIMES_D(s1,temp),SQR_QD(s2));\
                                }
#define DIV_QD_DATA(s1,s2,temp,out) {\
                                    CONJ_QD_DATA(s2,temp);\
                                    out.q[N]=_IQdiv(TIMES_Q_DATA(s1,temp,N),SQR_QD_NDATA(s2,N));\
                                    out.d[N]=_IQdiv(TIMES_D_DATA(s1,temp,N),SQR_QD_NDATA(s2,N));\
                                    out.q[N1]=_IQdiv(TIMES_Q_DATA(s1,temp,N1),SQR_QD_NDATA(s2,N1));\
                                    out.d[N1]=_IQdiv(TIMES_D_DATA(s1,temp,N1),SQR_QD_NDATA(s2,N1));\
                                    }

/*! @} */
#endif
//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

