/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name:   		adc.h
 Project:     		TIMotorLIB
 
 TI Doc: 		<a href="http://www.ti.com/lit/pdf/spru812"> SPRU812</a>
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \brief 		AD Routines
 \details		This module allows to program the AD converter. Supported
functions include the configuration of the channel list, read of values both in raw or
physical units and storage of readed values in memory.
 \date 			2007
******************************************************************************/

/** \example TIMotorLIBExample1/main.c
    * This is an example on how to use adc units.
    * The shown code adds two channels to de AD sequencer and reads the values
    * in the ISR.
*/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------
#ifndef ADC_H
#define ADC_H

//-----------------------------------------------------------------------------
//	Includes
//-----------------------------------------------------------------------------
#include "TIMotorLIB.h"
#include GENERATE_FLEX_INC(Device.h)
#include "IQConstants.h"
#include "DataStorage.h"

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

// acq system channels
#define MAX_CH		16
#define ADCINA0		0
#define ADCINA1		1
#define ADCINA2		2
#define ADCINA3		3
#define ADCINA4		4
#define ADCINA5		5
#define ADCINA6		6
#define ADCINA7		7
#define ADCINB0		8
#define ADCINB1		9
#define ADCINB2		10
#define ADCINB3		11
#define ADCINB4		12
#define ADCINB5		13
#define ADCINB6		14
#define ADCINB7		15
// Converter readings (two wait states, right justified)
#define CH0_RAW				((AdcRegs.ADCRESULT0 & 0xFFF0) >> 4)
#define CH1_RAW				((AdcRegs.ADCRESULT1 & 0xFFF0) >> 4)
#define CH2_RAW				((AdcRegs.ADCRESULT2 & 0xFFF0) >> 4)
#define CH3_RAW				((AdcRegs.ADCRESULT3 & 0xFFF0) >> 4)
#define CH4_RAW				((AdcRegs.ADCRESULT4 & 0xFFF0) >> 4)
#define CH5_RAW				((AdcRegs.ADCRESULT5 & 0xFFF0) >> 4)
#define CH6_RAW				((AdcRegs.ADCRESULT6 & 0xFFF0) >> 4)
#define CH7_RAW				((AdcRegs.ADCRESULT7 & 0xFFF0) >> 4)
#define CH8_RAW				((AdcRegs.ADCRESULT8 & 0xFFF0) >> 4)
#define CH9_RAW				((AdcRegs.ADCRESULT9 & 0xFFF0) >> 4)
#define CH10_RAW			((AdcRegs.ADCRESULT10 & 0xFFF0) >> 4)
#define CH11_RAW			((AdcRegs.ADCRESULT11 & 0xFFF0) >> 4)
#define CH12_RAW			((AdcRegs.ADCRESULT12 & 0xFFF0) >> 4)
#define CH13_RAW			((AdcRegs.ADCRESULT13 & 0xFFF0) >> 4)
#define CH14_RAW			((AdcRegs.ADCRESULT14 & 0xFFF0) >> 4)
#define CH15_RAW			((AdcRegs.ADCRESULT15 & 0xFFF0) >> 4)
// mirror result buffers (zero wait states, left justified SPRU716.pdf, pp:43) 
#define CH0_RAW_MIRROW 		(AdcMirror.ADCRESULT0)
#define CH1_RAW_MIRROW 		(AdcMirror.ADCRESULT1)
#define CH2_RAW_MIRROW 		(AdcMirror.ADCRESULT2)
#define CH3_RAW_MIRROW 		(AdcMirror.ADCRESULT3)
#define CH4_RAW_MIRROW 		(AdcMirror.ADCRESULT4)
#define CH5_RAW_MIRROW 		(AdcMirror.ADCRESULT5)
#define CH6_RAW_MIRROW 		(AdcMirror.ADCRESULT6)
#define CH7_RAW_MIRROW 		(AdcMirror.ADCRESULT7)
#define CH8_RAW_MIRROW 		(AdcMirror.ADCRESULT8)
#define CH9_RAW_MIRROW 		(AdcMirror.ADCRESULT9)
#define CH10_RAW_MIRROW 	(AdcMirror.ADCRESULT10)
#define CH11_RAW_MIRROW 	(AdcMirror.ADCRESULT11)
#define CH12_RAW_MIRROW 	(AdcMirror.ADCRESULT12)
#define CH13_RAW_MIRROW 	(AdcMirror.ADCRESULT13)
#define CH14_RAW_MIRROW 	(AdcMirror.ADCRESULT14)
#define CH15_RAW_MIRROW 	(AdcMirror.ADCRESULT15)
/// \def BITS2VOLTS(bits) bits --> volts conversion
#define BITS2VOLTS(bits)	((bits >> 12)*3 + ADCLO)
/// \def VOLTS2BITS(volts) volts --> bits conversion 
#define VOLTS2BITS(volts)	(4096/3*(volts - ADCLO))

#define AD_ORIGIN			(2048)


//-----------------------------------------------------------------------------
//	typedefs
// ----------------------------------------------------------------------------

/// \enum SAMPLING_MODE sampling mode configuration
typedef enum {
	CASCADED_MODE=0, 	///< AD reads in cascade
	SIMULTANEOUS_MODE=1	///< AD reads simultaneous
    } SAMPLING_MODE;
    
/// \enum SEQ_MODE sequence mode configuration
typedef enum {
	INDEPENDENT_SEQ_MODE=0, ///< Sequencers A/B are independent
	CASCADED_SEQ_MODE=1	///< Sequencers A/B are joined
    } SEQ_MODE;
    
/// \enum SOC_MODE start of conversion (SOC) configuration
typedef enum {
	PWM_SOC=(int)0x8901, 	///< Start of conversion triggered by PWM
	GPIO_SOC=0x0080, 	///< Start of conversion triggered by external GPIO
	SW_SOC=0x0000		///< Start of conversion triggered by software
    } SOC_MODE;	// <-- warning
    
/// \enum POLARITY input polarity configuration
typedef enum {
	UNIPOLAR=0,				///< Unipolar channel
	BIPOLAR=1				///< Bipolar channel
} POLARITY;

//-----------------------------------------------------------------------------
/// \struct ADConfiguration.
/// \brief storing of AD configuration data
// ----------------------------------------------------------------------------
struct ADConfiguration {
	Uint16 chlChannels[MAX_CH];	///< channel list
	Uint16 convChannel[MAX_CH];	///< channels per converter (can be 1 or 2 depending on independent/simultaneous)
	_iq chlGains[MAX_CH];		///< channel list gains
	_iq chlOffsets[MAX_CH];		///< channel list offset
	Uint16 chlPolarity[MAX_CH];	///< channel polarity (unipolar/bipolar)
	volatile Uint16* selSeqPtr;	///< sequence ptr
	Uint16 max_ch;				///< Maximum Conversion Channels Register
	volatile Uint16 trigger;	///< trigger software
	_iq fAD;			///< AD clock in Mhz
	Uint16 tSH;			///< SH acquisition time in us
};

extern struct ADConfiguration adConfiguration;




//-----------------------------------------------------------------------------
//	Implemented Functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Configure AD module.
///	\param interruptFCN pointer to interrupt routine called on A/D finished 
///	conversion. If not desired pass NULL.
/// \param fAD desired AD clock in Mhz
/// \param tSH desired SH acquisition time in ns
///	\param seqMode INDEPENDENT_SEQ_MODE or CASCADED_SEQ_MODE.
///	\param samplingMode CASCADED_MODE or SIMULTANEOUS_MODE
///	\param socMode: start of conversion modes
///		PWM_SOC (start of conversion by pwm), 
///		GPIO_SOC (start of conversion by external GPIO) or 
///		SW_SOC (start of conversion by software)
///	\param offtrim AD offset compensation
//-----------------------------------------------------------------------------
extern void ConfigureAD(void(*interruptFCN)(void), _iq fAD, Uint16 tSH, 
	SEQ_MODE seqMode, SAMPLING_MODE samplingMode, SOC_MODE socMode,
	Uint16 offtrim);

//-----------------------------------------------------------------------------
/// Add channel to the  AD module.
///	\param input input number (ADCINA0...ADCINA7, ADCINB0...ADCINB7)
///	\param channel set the S/H channel associated to the input.
///	\param offset offset in physical units
///	\param gain units/bits gain
//-----------------------------------------------------------------------------
extern void AddChannel(Uint16 input, Uint16 channel, _iq offset, _iq gain);

//-----------------------------------------------------------------------------
/// Add unipolar channel to the  AD module.
///	\param input input number (ADCINA0...ADCINA7, ADCINB0...ADCINB7)
///	\param channel set the S/H channel associated to the input.
///	\param offset offset in physical units
///	\param gain units/bits gain
//-----------------------------------------------------------------------------
extern void AddUnipolarChannel(Uint16 input, Uint16 channel, _iq offset, _iq gain);


//-----------------------------------------------------------------------------
/// Initializes AD circuits.
//-----------------------------------------------------------------------------
extern void InitAD(void);

//-----------------------------------------------------------------------------
/// Initializes AD clock
//-----------------------------------------------------------------------------
extern void InitADPeripheralClocks(void);

//-----------------------------------------------------------------------------
/// Prepare A/D subsystem SEQ1 for a next adquisition. 
/// This function must be called after all the readings in
/// the actual sampling period are done.
//-----------------------------------------------------------------------------
static inline void ReinitADSEQ1(void);

//-----------------------------------------------------------------------------
/// Prepare A/D subsystem SEQ2 for a next adquisition. 
/// This function must be called after all the readings in
/// the actual sampling period are done.
//-----------------------------------------------------------------------------
static inline void ReinitADSEQ2(void);

//-----------------------------------------------------------------------------
/// Trigger to enable internal memory writing.
//-----------------------------------------------------------------------------
static inline void SetTrigger(void);

//-----------------------------------------------------------------------------
/// Reset trigger for internal memory writing.
//-----------------------------------------------------------------------------
static inline void ResetTrigger(void);


//-----------------------------------------------------------------------------
/// Read AD channel in raw (bits).
///	\param channel channel to be read.
///	\return raw value (bits)
//-----------------------------------------------------------------------------
static inline Uint16 ReadADChannelRaw(Uint16 channel);
						
//-----------------------------------------------------------------------------
/// Read AD input in raw (bits).
///	\param input input to be read.
///	\return raw value (bits)
//-----------------------------------------------------------------------------
static inline Uint16 ReadADInputRaw(Uint16 input);

//-----------------------------------------------------------------------------
/// Read AD channel in raw (bits) and write the value to internal memory.
///	\param channel channel to be read.
///	\return raw value (bits)
//-----------------------------------------------------------------------------
static inline Uint16 ReadADChannelRawWriteRaw(Uint16 channel);
						
//-----------------------------------------------------------------------------
/// Read AD input in raw (bits) and write the value to internal memory.
///	\param input channel to be read.
///	\return raw value (bits)
//-----------------------------------------------------------------------------
static inline Uint16 ReadADInputRawWriteRaw(Uint16 input);
		
//-----------------------------------------------------------------------------
/// Read AD input in units.
///	\param input input to be read.
///	\return readed value in physical units
//-----------------------------------------------------------------------------
static inline _iq ReadADInput(Uint16 input);
				
//-----------------------------------------------------------------------------
/// Read AD input in units and write raw (bits) value to internal memory.
///	\param input input to be read.
///	\return readed value in physical units
//-----------------------------------------------------------------------------
static inline _iq ReadADInputWriteRaw(Uint16 input);

//-----------------------------------------------------------------------------
/// Read two AD simulatenous inputs in units and write raw (bits) value to internal memory.
///	\param aInput a input to be read.
///	\param bInput b input to be read.
///	\param aVal readed value for a input in physical units
///	\param bVal readed value for b input in physical units
//-----------------------------------------------------------------------------
static inline void ReadADSimultaneousInputWriteRaw(Uint16 aInput, Uint16 bInput,
	 volatile _iq* aVal, volatile _iq* bVal);
	 
	 
//-----------------------------------------------------------------------------
///	Convert AD read from bits to units
/// \param rawVal readed value in bits
/// \param input readed input
/// \return value in units 
//-----------------------------------------------------------------------------
static inline _iq Bits2Units(Uint16 rawVal, Uint16 input);							


//-----------------------------------------------------------------------------
//	inlined functions implementation
//-----------------------------------------------------------------------------


/// Prepare AD SEQ1 for next acquisition: Reset SEQ1, clear INT SEQ1, ACK int to PIE 
static inline void ReinitADSEQ1(void)
{
	// Reinitialize for next ADC sequence
  	AdcRegs.ADCTRL2.bit.RST_SEQ1 = 1;         	// Reset SEQ1
  	AdcRegs.ADCST.bit.INT_SEQ1_CLR = 1;       	// Clear INT SEQ1 bit
  	PieCtrlRegs.PIEACK.all |= PIEACK_GROUP1;   // Acknowledge interrupt to PIE
}

//-----------------------------------------------------------------------------

/// Prepare AD SEQ2 for next acquisition: Reset SEQ2, clear INT SEQ2, ACK int to PIE 
static inline void ReinitADSEQ2(void)
{
	// Reinitialize for next ADC sequence
  	AdcRegs.ADCTRL2.bit.RST_SEQ2 = 1;         	// Reset SEQ2
  	AdcRegs.ADCST.bit.INT_SEQ2_CLR = 1;       	// Clear INT SEQ2 bit
  	PieCtrlRegs.PIEACK.all |= PIEACK_GROUP1;   // Acknowledge interrupt to PIE
}

//-----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// Set trigger equal 1. It disables internal memory writing of AD readed values
// ----------------------------------------------------------------------------
static inline void SetTrigger(void)
{
	adConfiguration.trigger=1;
}

//-----------------------------------------------------------------------------



// ----------------------------------------------------------------------------
/// Set trigger equal 0. It disables internal memory writing of AD readed values
// ----------------------------------------------------------------------------
static inline void ResetTrigger(void)
{
	adConfiguration.trigger=0;
}

//-----------------------------------------------------------------------------



// ----------------------------------------------------------------------------
/// Read raw channel (possition in CONV)
// ----------------------------------------------------------------------------
static inline Uint16 ReadADChannelRaw(Uint16 channel)
{
	return *(&AdcMirror.ADCRESULT0 + channel);
}

//-----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// Read raw input (physical ADINPUT)
// ----------------------------------------------------------------------------
static inline Uint16 ReadADInputRaw(Uint16 input)
{
	return *(&AdcMirror.ADCRESULT0 + adConfiguration.chlChannels[input]);
}

//-----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// Read channel in raw and writes raw values
// ----------------------------------------------------------------------------
static inline Uint16 ReadADChannelRawWriteRaw(Uint16 channel)
{
	Uint16 rawVal=ReadADInputRaw(channel);
	#ifdef MEM_STORAGE_DEFINED
	if(adConfiguration.trigger){
		WriteData(&rawVal);
   	}
	#endif
	return rawVal;
}

//-----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// Read input in raw and writes raw values
// ----------------------------------------------------------------------------
static inline Uint16 ReadADInputRawWriteRaw(Uint16 input)
{
	Uint16 rawVal=ReadADInputRaw(input);
	#ifdef MEM_STORAGE_DEFINED
	if(adConfiguration.trigger){
		WriteData(&rawVal);
   	}
	#endif
	return rawVal;
}

//-----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
/// Read channel with specified gain and offset
// ----------------------------------------------------------------------------
static inline _iq ReadADInput(Uint16 input)
{
	Uint16 rawVal=ReadADInputRaw(input);
   	return Bits2Units(rawVal, input);
}

//-----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// Read channel with specified gain and offset and write raw to mem
// ----------------------------------------------------------------------------
static inline _iq ReadADInputWriteRaw(Uint16 input)
{
	Uint16 rawVal=ReadADInputRaw(input);
	#ifdef MEM_STORAGE_DEFINED
	if(adConfiguration.trigger){
		WriteData(&rawVal);
   	}
	#endif
    return Bits2Units(rawVal, input);
}

//-----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
/// Read two simultaneous channel. Input is the first channel. Values are returned
///	on aVal, bVal
// ----------------------------------------------------------------------------
static inline void ReadADSimultaneousInputWriteRaw(Uint16 aInput, Uint16 bInput,
	 volatile _iq* aVal, volatile _iq* bVal)
{
	Uint16 rawaVal = ReadADInputRaw(aInput);
	Uint16 rawbVal = ReadADInputRaw(bInput);
	#ifdef MEM_STORAGE_DEFINED
	if(adConfiguration.trigger){
		if(dataPtr+2 < end){
			WriteData(&rawaVal);
			WriteData(&rawbVal);
		}
   	}
	#endif
	*aVal = Bits2Units(rawaVal, aInput);
	*bVal = Bits2Units(rawbVal, bInput);
}

//-----------------------------------------------------------------------------

static inline _iq Bits2Units(Uint16 rawVal, Uint16 input)
{
	return _IQmpy( ((long)(rawVal-adConfiguration.chlPolarity[input]))<<GLOBAL_Q, 
				 adConfiguration.chlGains[input]) + 		   	
   				 adConfiguration.chlOffsets[input];
}

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------


