/******************************************************************************
** Copyright (C) 2010  
** Silvia Izquierdo Rosas <silvia.izquierdo.rosas@gmail.com>
** Christian Blanco Charro <khristian996@gmail.com>
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name:   		spi.h
 Project:     		TIMotorLIB
 
 TI Doc: 		<a href="http://focus.ti.com/lit/ug/sprueu3a/sprueu3a.pdf"> SPRUEU3A</a>
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2011	      
 \brief 		SPI Routines
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------
#ifndef SPI_H
#define SPI_H

//-----------------------------------------------------------------------------
//	typedefs
// ----------------------------------------------------------------------------

/// SPI Network Mode Control
typedef enum {
	SPISLAVE=0, 	///< SPI configured as a slave
	SPIMASTER=1		///< SPI configured as a master
    } SPINMC;

/// Loopback mode
typedef enum{
	SPILBKDIS=0, 	///< SPI disable loopback
	SPILBKENA=1 	///< SPI enable loopback
	} SPILBK;

// ----------------------------------------------------------------------------
//Implemented Functions
// ----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Intializes SPIA
/// \param interruptRX: pointer to ISR executed when new data is received
/// \param interruptTX: pointer to ISR executed when there is enough room in
/// the txFIFO.
/// \param networkModeControl: device as master or slave (SPIMASTER/SPISLAVE)
/// \param spiLoopBack: enable/disable loopback mode (SPILBKENA/SPILBKDIS)
/// \param speed: baud rate configuration in Mhz
/// \param SPIcharLength: Character Length Control. Number of bits to be shifted in or
/// out as a single character during one shift sequence.
/// \param txFIFO: transmit FIFO interrupt level bits. 
/// \param rxFIFO: receive FIFO interrupt level bits. 
//-----------------------------------------------------------------------------
void ConfigureSPIA(void(*interruptRX)(void), void(*interruptTX)(void), 
	SPINMC networkModeControl, SPILBK spiLoopBack, Uint32 speed, 
	Uint16 SPIcharLength, Uint16 txFIFO, Uint16 rxFIFO);
void InitSpiaGpio(void);

static inline void StartTransmission(void);
static inline void EndTransmission(void);
static inline void SPIATxPIEAck(void);
static inline void SPIARxPIEAck(void);
static inline void SendDataCharSpiA(volatile char *a);
static inline void SendDataCharSpiANow(volatile char *a);
static inline void SendDataIntSpiA(volatile Uint16 *a);
static inline void SendDataIntSpiANow(volatile Uint16 *a);
static inline void SendDataIQSpiA(volatile Uint32 *a);
static inline void SendDataIQSpiANow(volatile Uint32 *a);

// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// static inline functions
// ----------------------------------------------------------------------------


static inline void StartTransmission(void)
{
	SpiaRegs.SPICCR.bit.SPISWRESET=1;  // Enable SPI
	SpiaRegs.SPIFFTX.bit.TXFIFO=1;
	SpiaRegs.SPIFFRX.bit.RXFIFORESET=1;
}

// ----------------------------------------------------------------------------


static inline void EndTransmission(void)
{
	SpiaRegs.SPICCR.bit.SPISWRESET=0;  // Disable SPI
	SpiaRegs.SPIFFTX.bit.TXFIFO=0;
	SpiaRegs.SPIFFRX.bit.RXFIFORESET=0;
}

// ----------------------------------------------------------------------------

static inline void SPIATxPIEAck(void)
{
    SpiaRegs.SPIFFTX.bit.TXFFINTCLR=1;  // Clear Interrupt flag
	PieCtrlRegs.PIEACK.all|=0x20;  		// Issue PIE ACK
}

// ----------------------------------------------------------------------------

static inline void SPIARxPIEAck(void)
{
	SpiaRegs.SPIFFRX.bit.RXFFOVFCLR=1;  // Clear Overflow flag
	SpiaRegs.SPIFFRX.bit.RXFFINTCLR=1; 	// Clear Interrupt flag
	PieCtrlRegs.PIEACK.all|=0x20;       // Issue PIE ack
}

// ----------------------------------------------------------------------------

static inline void SendDataCharSpiA(volatile char *a)
{
	//Receives a pointer to the number  
	while (SpiaRegs.SPIFFTX.bit.TXFFST != 0) {}	// si TXFFST=0 -> fifo is empty, ready to transmit TODO: Pablo Interrupts??
	SpiaRegs.SPITXBUF = *a;	
}

// ----------------------------------------------------------------------------

static inline void SendDataIntSpiA(volatile Uint16 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<2;i++) {	
	    while (SpiaRegs.SPIFFTX.bit.TXFFST != 0) {}	// si TXFFST=0 -> fifo is empty, ready to transmit TODO: Pablo Interrupts??	    
	    SpiaRegs.SPITXBUF = (*a >> 8*i & 0x00FF);
	}
}

// ----------------------------------------------------------------------------

static inline void SendDataIQSpiA(volatile Uint32 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<4;i++) {	
	    while (SpiaRegs.SPIFFTX.bit.TXFFST != 0) {}	// si TXFFST=0 -> fifo is empty, ready to transmit TODO: Pablo Interrupts??	    
	    SpiaRegs.SPITXBUF = (*a >> 8*i & 0x00FF);
	}	
}

// ----------------------------------------------------------------------------

static inline void SendDataCharSpiANow(volatile char *a)
{
	//Receives a pointer to the number  
	SpiaRegs.SPITXBUF = *a;	
}

// ----------------------------------------------------------------------------

static inline void SendDataIntSpiANow(volatile Uint16 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<2;i++) {	
	    SpiaRegs.SPITXBUF = (*a >> 8*i & 0x00FF);
	}
}

// ----------------------------------------------------------------------------

static inline void SendDataIQSpiANow(volatile Uint32 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<4;i++) {
	    SpiaRegs.SPITXBUF = (*a >> 8*i & 0x00FF);
	}	
}

// ----------------------------------------------------------------------------

#endif
//-----------------------------------------------------------------------------
// End Of File
//----------------------------------------------------------------------------- 
